// ********************************************************************
//
// source:
//
// type:      source code
//
// created:   31. Aug 2001
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: A class to fit primary vertices by using the SVertex package
//
// changes:
// 31 Aug 2001 (TG) creation
// 04 Jan 2002 (TG) added #define PRIMARYVERTEX_CODE to avoid that a huge
//                  amount of code is generated during compilation by the
//                  SVertex::isValid() routine.
//
// ********************************************************************

#include <iostream>
#define PRIMARYVERTEX_CODE
#include "clue/PrimaryVertex.hh"
#include "pattern/SmartPointer.hh"
#include "vt++/VtVertex.hh"

//==============================================================================
// Constructors
//==============================================================================
PrimaryVertex::PrimaryVertex() : 
  tracks()
{}

PrimaryVertex::~PrimaryVertex() {}

//==============================================================================
// meta_for_fill_vtx
//==============================================================================
template <unsigned int I>
struct meta_for_fill_vtx {
  template <unsigned int NTR>
  inline static void f(SVertex<NTR>& vtx, const vector<TrackIf*>& vec) {
    vtx.track(I) = vec[I];
    meta_for_fill_vtx<I-1>::f(vtx,vec);
  }
};

//==============================================================================
// meta_for_fill_vtx
//==============================================================================
template<>
struct meta_for_fill_vtx<0> {
  template <unsigned int NTR>
  inline static void f(SVertex<NTR>& vtx, const vector<TrackIf*>& vec) {
    vtx.track(0) = vec[0];
  }
};

//==============================================================================
// findVertexVt()
//==============================================================================
bool PrimaryVertex::findVertexVt() {

  const unsigned int SIZE = tracks.size();
  bool rc = false;
  switch(SIZE) {
  case 2:
    {
      static SVertex<2>  vtx2;
      meta_for_fill_vtx<1>::f(vtx2, tracks);
      rc = vtx2.findVertexVt();
      vtx = vtx2;
    }
    break;
  case 3:
    {
      static SVertex<3>  vtx3;
      meta_for_fill_vtx<2>::f(vtx3, tracks);
      rc = vtx3.findVertexVt();
      vtx = vtx3;
    }
    break;
  case 4:
    {
      static SVertex<4> vtx4;
      meta_for_fill_vtx<3>::f(vtx4, tracks);
      rc = vtx4.findVertexVt();
      vtx = vtx4;
    }
    break;
  case 5:
    {
      static SVertex<5>  vtx5;
      meta_for_fill_vtx<4>::f(vtx5, tracks);
      rc = vtx5.findVertexVt();
      vtx = vtx5;
    }
    break;
  case 6:
    {
      static SVertex<6>  vtx6;
      meta_for_fill_vtx<5>::f(vtx6, tracks);
      rc = vtx6.findVertexVt();
      vtx = vtx6;
    }
    break;
  case 7:
    {
      static SVertex<7>  vtx7;
      meta_for_fill_vtx<6>::f(vtx7, tracks);
      rc = vtx7.findVertexVt();
      vtx = vtx7;
    }
    break;
  case 8:
    {
      static SVertex<8>  vtx8;
      meta_for_fill_vtx<7>::f(vtx8, tracks);
      rc = vtx8.findVertexVt();
      vtx = vtx8;
    }
    break;
  case 9:
    {
      static SVertex<9>  vtx9;
      meta_for_fill_vtx<8>::f(vtx9, tracks);
      rc = vtx9.findVertexVt();
      vtx = vtx9;
    }
    break;
  case 10:
    {
      static SVertex<10> vtx10;
      meta_for_fill_vtx<9>::f(vtx10, tracks);
      rc = vtx10.findVertexVt();
      vtx = vtx10;
    }
    break;
  case 11:
    {
      static SVertex<11> vtx11;
      meta_for_fill_vtx<10>::f(vtx11, tracks);
      rc = vtx11.findVertexVt();
      vtx = vtx11;
    }
    break;
  case 12:
    {
      static SVertex<12> vtx12;
      meta_for_fill_vtx<11>::f(vtx12, tracks);
      rc = vtx12.findVertexVt();
      vtx = vtx12;
    }
    break;
  case 13:
    {
      static SVertex<13> vtx13;
      meta_for_fill_vtx<12>::f(vtx13, tracks);
      rc = vtx13.findVertexVt();
      vtx = vtx13;
    }
    break;
  case 14:
    {
      static SVertex<14> vtx14;
      meta_for_fill_vtx<13>::f(vtx14, tracks);
      rc = vtx14.findVertexVt();
      vtx = vtx14;
    }
    break;
  case 15:
    {
      static SVertex<15> vtx15;
      meta_for_fill_vtx<14>::f(vtx15, tracks);
      rc = vtx15.findVertexVt();
      vtx = vtx15;
    }
    break;
  case 16:
    {
      static SVertex<16> vtx16;
      meta_for_fill_vtx<15>::f(vtx16, tracks);
      rc = vtx16.findVertexVt();
      vtx = vtx16;
    }
    break;
  case 17:
    {
      static SVertex<17> vtx17;
      meta_for_fill_vtx<16>::f(vtx17, tracks);
      rc = vtx17.findVertexVt();
      vtx = vtx17;
    }
    break;
  case 18:
    {
      static SVertex<18> vtx18;
      meta_for_fill_vtx<17>::f(vtx18, tracks);
      rc = vtx18.findVertexVt();
      vtx = vtx18;
    }
    break;
  case 19:
    {
      static SVertex<19> vtx19;
      meta_for_fill_vtx<18>::f(vtx19, tracks);
      rc = vtx19.findVertexVt();
      vtx = vtx19;
    }
    break;
  case 20:
    {
      static SVertex<20> vtx20;
      meta_for_fill_vtx<19>::f(vtx20, tracks);
      rc = vtx20.findVertexVt();
      vtx = vtx20;
    }
    break;
  default:
    {
      static VERTEX::Vertex ver;
      ver.clear();
      for(unsigned int i=0; i<SIZE; ++i) {
	ver.push_back(*tracks[i]);
      }
      rc = ver.findVertexVt();
      vtx = ver;
    }
    break;
  } // switch

  return rc;
}


//==============================================================================
// VertexIf access methods
//==============================================================================
float PrimaryVertex::vx()   const { return vtx.vx(); }
float PrimaryVertex::vy()   const { return vtx.vy(); }
float PrimaryVertex::vz()   const { return vtx.vz(); }
float PrimaryVertex::chi2() const { return vtx.chi2(); }
float PrimaryVertex::prob() const { return vtx.prob(); }

unsigned short int PrimaryVertex::ndf()     const { return vtx.ndf(); }
unsigned short int PrimaryVertex::ntracks() const { return vtx.ntracks(); }

const TrackIf* PrimaryVertex::track(unsigned int i) const { return vtx.track(i); }
const TrackIf*& PrimaryVertex::track(unsigned int i) { return vtx.track(i); }

SVector<double,3> PrimaryVertex::vpos() const { return vtx.vpos(); }
const SMatrix<double,3>& PrimaryVertex::VCOV() const { return vtx.VCOV(); }

float PrimaryVertex::vtx_cov_x()   const { return vtx.vtx_cov_x(); }
float PrimaryVertex::vtx_cov_y()   const { return vtx.vtx_cov_y(); }
float PrimaryVertex::vtx_cov_z()   const { return vtx.vtx_cov_z(); }

std::ostream& PrimaryVertex::print( std::ostream& os ) const 
{ return vtx.print(os); }

#undef PRIMARYVERTEX_CODE
