//**********************************************************************
//
// source:
//
// type:      source code
//
// created:   11. Mai 2000
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: Class for a HERA-B wire target.
//
// changes:
// 11 Mai 2000 (TG) creation
// 03 Nov 2000 (TG) removed #ifdef __CINT__, changed include files
// 20 Nov 2000 (TG) added namespace CLUE
// 12 Jan 2001 (TG) modified constructors, changed print()
//
//**********************************************************************

#include <iostream>
#include <iomanip>

#include "clue/Wire.hh"
#include "geometry/Point3D.hh"

#ifdef USE_ROOT
#include <TPad.h>
#endif

#ifdef NAMESPACE
namespace CLUE {
#endif

  //==============================================================================
  // Constructors
  //==============================================================================
  Wire::Wire() : X(0.), Y(0.), Z(0.), dX(0.), dY(0.), dZ(0.) {}

  Wire::Wire( const Wire& rhs ): 
    X ( rhs.X ),
    Y ( rhs.Y ),
    Z ( rhs.Z ),
    dX( rhs.dX ),
    dY( rhs.dY ),
    dZ( rhs.dZ )
    {}
  
  Wire::~Wire() {}
  
  
  //==============================================================================
  // print
  //==============================================================================
  std::ostream& Wire::print( std::ostream& os) const {
    os.setf(ios::right,ios::adjustfield);
    os.setf(ios::fixed,ios::floatfield);
    os << "Wire: "
       << " Center: "    
       << setw(5) << setprecision(2) << x() << "," 
       << setw(5) << setprecision(2) << y() << "," 
       << setw(5) << setprecision(2) << z()
       << "\t"
       << " Thickness: " 
       << setw(5) << setprecision(2) << dx() << "," 
       << setw(5) << setprecision(2) << dy() << "," 
       << setw(5) << setprecision(2) << dz();
    return os;
  }
  
  
#ifdef USE_ROOT
  //==============================================================================
  // Print
  //==============================================================================
  void Wire::Print(Option_t * ) {
    cout << *this << endl;
    return;
  }
  
  //==============================================================================
  // Root::Paint
  //==============================================================================
  // How to use: a) create canvas 
  //             b) create view via:  TView* view = new TView(1)
  //             c) loop over tracks and call Draw()
  // 
  void Wire::Paint(Option_t *option) {
    const Point3D Center(x(),y(),z());
    const Point3D Thickness(dx(), dy(), dz());
    const Point3D x( Center - 0.5 * Thickness );
    const Point3D y( Center + 0.5 * Thickness );

    Float_t xx[3] = { x.x(), x.y(), x.z() };
    Float_t yy[3] = { y.x(), y.y(), y.z() };
    
    gPad->PaintLine3D( xx, yy );
  }
  
  ClassImp(Wire)
#endif
#ifdef NAMESPACE
} // namespace CLUE
#endif
