#ifndef __CLUETTREE_HH
#define __CLUETTREE_HH
// ********************************************************************
//
// source:
//
// type:      source code
//
// created:   14. Dez 2000
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: My own TTree object
//
// changes:
// 14 Dec 2000 (TG) creation
// 12 Nov 2001 (TG) comments improved
//
// ********************************************************************
#ifdef USE_ROOT
#include <TTree.h>

/** Modified ROOT TTree class.

   This TTree class fixes a bug in ROOT versions $\le$ 2.25.03 which is
   essential for the STL compatible TClonesArrays of the \texttt{Gate}
   project. Since ROOT version 3.00 this bug is fixed in the official ROOT
   version.

   @author T. Glebe
*/
//==============================================================================
// ClueTTree
//==============================================================================
class ClueTTree: public TTree {
public:
  /** @name --- Constructors --- */
  ///
  ClueTTree() : TTree() {}
  ///
  ClueTTree(const char *name, const char *title, Int_t maxvirtualsize=0) :
    TTree(name, title, maxvirtualsize) {}
  ///
  virtual ~ClueTTree() {}
  
  /** @name --- Access methods --- */
  /// fixed TTree function
  virtual TBranch  *Branch(const char *name, const char *classname,
			   void *addobj, Int_t bufsize=32000, Int_t splitlevel=1);

};
#endif  //USE_ROOT
#endif
