#ifndef PKG_INI_HH
#define PKG_INI_HH
// ********************************************************************
//
// source:
//
// type:      source code
//
// created:   18. Dec 2000
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: Class to initialize a BEE project
//
// changes:
// 18 Dec 2000 (TG) creation
// 19 Dec 2000 (TG) moved ROOT initialization to ClueIni.hh
//
// ********************************************************************
#include <iosfwd>

#ifdef NAMESPACE
namespace CLUE {
#endif
  class abc;  // to fake doc++

  /**
     Class to hold the XCVS information of a BEE package. Each project should
     create a global PkgIni object which is initialized with the projects XCVS
     information.
     @memo Initialization of BEE package
     @author T. Glebe
  */
  //==============================================================================
  // ClueIni
  //==============================================================================
  class PkgIni {
  public:
    /** @name --- Constructors --- */
    /// for ROOT backward compatibility
    PkgIni();
    ///
    PkgIni( const char* nam, const char* vers, const char* dat );
    ///
    ~PkgIni();

    /** @name --- Access methods ---*/
    /// get string with version number
    const char* const version() const;
    /// get string with package name
    const char* const name()    const;
    /// get birthday \& time of package version
    const char* const date()    const;
    /// toggle debug flag
    const bool debug();
    /// return debug flag
    bool isDebug()              const;

    /** @name --- Expert functions ---*/
    /// used by operator<<()
    std::ostream& print(std::ostream& os) const;
    
  private:
    const char *Name;
    const char *Version;
    const char *Date;
    bool        Debug;

  }; // end of class PkgIni
  
  
  //============================================================================
  // operator<<
  //============================================================================
  std::ostream& operator<<( std::ostream& os, const PkgIni& t );

#ifdef NAMESPACE
} // namespace CLUE
#endif
#ifndef __CINT__
#include "PkgIni.icc"
#endif
#endif
