// **********************************************************************
//
// source:
//
// type:      source code
//
// created:   20. Mai 2000
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: Class to initialize the Clue library
//
// changes:
// 20 Mai 2000 (TG) creation
// 20 Nov 2000 (TG) added namespace CLUE
//
// **********************************************************************

#include <iostream>
#include "clue/CVStag.Clue"
#include "clue/ClueIni.hh"

#ifdef USE_ROOT
#include "TROOT.h"
#endif

#ifdef NAMESPACE
namespace CLUE {
#endif

  ClueIni::initializer ClueIni::doIni;
  
  const char *ClueIni::Version = xcvsVERSION_Clue;
  const char *ClueIni::Name    = xcvsMODULE_Clue;
  const char *ClueIni::Date    = xcvsDAYTIME_Clue;
  bool        ClueIni::Debug   = false;

  //==============================================================================
  // Constructors
  //==============================================================================
  ClueIni::ClueIni() {}
  
  //==============================================================================
  // Constructor for private class initializer
  //==============================================================================
  ClueIni::initializer::initializer() {
    cout << xcvsMODULE_Clue 
	 << " version " << xcvsVERSION_Clue 
	 << " loaded (" << xcvsDAYTIME_Clue << ")."
	 << endl;
    
#ifdef USE_ROOT
    if(!gROOT) {
      cout << "====> ClueIni: INITIALIZE ROOT SYSTEM!" << endl;
      TROOT root("BEE", "The HERA-B analysis framework");
    }
#endif
  }
  
  //============================================================================
  // ClueIni::print
  //============================================================================
  void ClueIni::print(std::ostream& os) const {
    cout << name()
	 << " version " << version()
	 << " loaded (" << date() << ")."
	 << endl;
  }
  
  //==============================================================================
  // debug
  //==============================================================================
  const bool ClueIni::debug() {
    return Debug = ((Debug==true) ? false : true); 
  }

#ifdef NAMESPACE
} // end of namespace CLUE
#endif
