#ifndef CLUE_UTIL_HH
#define CLUE_UTIL_HH
// *****************************************************************************
//
// source:
//
// type:      source code
//
// created:   22 Mai 2000
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: 
//
// changes:
// 22 Mai 2000 (TG) creation
// 16 Nov 2000 (TG) moved getPTR(), getIDX() to arte/arte.hh
// 20 Nov 2000 (TG) added namespace CLUE
// 12 Jan 2001 (TG) added WireDirection(), Propagate()
//
// *****************************************************************************

#include <iosfwd>
#include "geometry/geomfwd.hh"
#include "clue/cluefwd.hh"

#ifdef NAMESPACE
namespace CLUE {
#endif
  class abc; // to fake doc++

  /** @name --- Global Utility Functions --- */
  //==============================================================================
  // WireDirection
  //==============================================================================
  /// return unit vector of Wire direction
  const ThreeVector WireDirection( const Wire& rhs );

  //==============================================================================
  // Propagate
  //==============================================================================
  /// calculate impact point at z
  const Point3D Propagate(const RecoSegment& rs, float z);

  //==============================================================================
  // distance:
  //==============================================================================
  /// dist between two rays (unsigned)
  const double distance(const ThreeVector& r1, const ThreeVector& m1,
			const ThreeVector& r2, const ThreeVector& m2);
  
  /// distance between RecoSegment and Wire
  const double distance( const RecoSegment& rseg, const Wire& wire);
  /// distance between RecoSegment and Wire
  const double distance( const Wire& wire, const RecoSegment& rseg); 

#ifdef NAMESPACE
} // namespace CLUE
#endif
#endif
