
/* ********************************************************************/
//
// $Source: /home/cluerep/CVS/Clue/old/SmartPointer.hh,v $
// $Revision: 1.1.1.36 $
// $Author: cluerep $
// $Date: 2002/01/18 15:52:29 $
//
// Created: 07-22-97,   C. Borgmeier
//
// class SmartPointer
//
// hm
//
/* ********************************************************************/


#ifndef SMART_POINTER_HH
#define SMART_POINTER_HH

#if !CINTDICT
#include <iostream.h>
#endif

#ifdef NAMESPACE
namespace CLUE {
#endif

  template<class t> class SmartPointer;

  /** Wrapper should be a private subclass of SmartPointer, but due to
      limitations of g++ this is not possible.
      {\bf Do not use it directly.}
      @memo only used in \Ref{SmartPointer}
      @author C. Borgmeier
  */
  //==============================================================================
  // Wrapper
  //==============================================================================
  template<class t>
    class Wrapper {
      friend class SmartPointer<t>;
    public:
      ///
      Wrapper( const t& obj);
      ///
      Wrapper( t* obj);
      ///
      ~Wrapper();
    private:
      t* object;
      unsigned int count;
    }; // end of class wrapper
  
  
#if !CINTDICT
  
  template<class t>
    /** Reference counting pointer. Not available in CINT.
	@author C. Borgmeier
    */
    //==============================================================================
    // SmartPointer used in Pattern
    //==============================================================================
    class SmartPointer {
    public:
      /// default CTOR
      inline SmartPointer();
      /// takes object ownership
      inline SmartPointer( t* obj );
      /// copies object
      inline SmartPointer( t const& obj );
      /// copies pointer
      inline SmartPointer( SmartPointer<t> const& arg );
      /// object is deleted together with the last pointer to it
      inline ~SmartPointer();
      
      /// assignment operator for reference counting
      inline SmartPointer<t>& operator=( SmartPointer<t> const& arg );
      ///
      t& operator*() { return *(pointer->object); }
      ///
      t const& operator*() const { return *(pointer->object); }
      
      /// passes {\tt operator<} to the object
      inline int operator<( SmartPointer<t> const& arg ) const;
      
      /// used by operator<<()
      inline void print( std::ostream& os ) const;
      
    private:
      Wrapper<t>* pointer;
    }; // end of class SmartPointer
  
  //==============================================================================
  // operator <<
  //==============================================================================
  template<class t>
    inline std::ostream& operator<<( std::ostream& os, SmartPointer<t> const& p );
  
#include "SmartPointer.icc"
#endif // ! CINTDICT

#ifdef NAMESPACE
} // namespace CLUE  
#endif
#endif
