// ********************************************************************
//
// source:
//
// type:      source code
//
// created:   Fre Sep 28 10:39:20 MEST 2001
//
// author:    glebe
//
// Description: 
//
// changes:
//
// ********************************************************************
#include <iostream>
#include "TH1.h"
#include "TH2.h"
#include "TNtuple.h"
#include "TPaveText.h"
#include "BEE.hh"
#include "include/K0Analysis.hh"

//==============================================================================
// Constructor
//==============================================================================
K0Analysis::K0Analysis(RtEvent* evt) :
 event(evt),
 Piplus(false),
 Piminus(false),
 kshort(false)
{}

//==============================================================================
// EventSelect
//==============================================================================
bool K0Analysis::EventSelect() {
  return true;
}

//==============================================================================
// DefinePattern
//==============================================================================
bool K0Analysis::DefinePattern() {

  Piminus    = pi_minus.createPattern();
  Piplus     = pi_plus.createPattern();

  kshort = K_zero.decaysTo(Piminus & Piplus);
  kshort->with(prob > 0.001 & vz > 0.);

  return true;
}

//==============================================================================
// DefineHistograms
//==============================================================================
bool K0Analysis::DefineHistograms() {

  Kshort = new TNtuple("Kshort","K0s","m");

  return true;
}

//==============================================================================
// Event
//==============================================================================
bool K0Analysis::Event() {

  // clear Particle container
  Piplus->clear();
  Piminus->clear();

  //----------------- select Particles -------------------
  const ArteTableIterator<RecoTrack> RTend = ArteTable<RecoTrack>::end();
  for(ArteTableIterator<RecoTrack> RTit = ArteTable<RecoTrack>::begin();
      RTit != RTend; ++RTit) {
    RecoTrack& rtra = **RTit;

    if(rtra.pid() != 0 && rtra.pid() != PID::pion) continue;
    if(rtra.hsi() < 6 ||
       (rtra.hou() + rtra.hin() < 16)) continue;
    if(rtra.pt() < 0.1 || rtra.p() < 2. || fabs(rtra.xf(pi_plus.mass())) > 0.2)
      continue;

    // prepare track for vertexing
    rtra.PrepareForVertexing();
    const int chrg = rtra.charge();

    if(chrg > 0) {
      Piplus->push_back(rtra);
      //      cout << " pi+ found " << endl;
    }
    if(chrg < 0) {
      Piminus->push_back(rtra);
      //      cout << " pi- found " << endl;
    }
  } // for RecoTrack


  //------------------ Pattern ----------------------
  kshort->combine();
  kshort->prob_collision();

  //------------ Loop over Pattern Matches -----------
  const TwoProngCandidateConstIterator end = kshort->matches().end();
  for(TwoProngCandidateConstIterator it = kshort->matches().begin();
      it != end; ++it) {
    const TwoProngCandidate cand = *it;

    if(!cand->isValid()) continue;

    float x[1];
    x[0] = cand->mass();

    Kshort->Fill(x);
  } // for SDPM

  return true;
}

//==============================================================================
// EventCleanup
//==============================================================================
bool K0Analysis::EventCleanup() {
  return true;
}

//==============================================================================
// End processing
//==============================================================================
bool K0Analysis::End() {
  return true;
}

