// ********************************************************************
//
// source:
//
// type:      source code
//
// created:   Die Aug 14 10:33:50 MEST 2001
//
// author:    glebe
//
// Description: Kshort decays are selected from MC truth (ArteTable<MC_Track>).
//              The found decay chains are filled into the pattern match
//              container (PiPlus, PiMinus, K_short).
//
// changes:
// 14 Sep 2001 (TG) renamed sdecaysTo() to decaysTo()
//
// ********************************************************************

#include "TH1.h"
#include "TH2.h"
#include "TNtuple.h"
#include "TPaveText.h"
#include "BEE.hh"
#include "include/MCAna.hh"

//==============================================================================
// Constructor
//==============================================================================
MCAna::MCAna(RtEvent* evt) :
 event(evt),
 PiPlus(false),
 PiMinus(false),
 K_short(false)
{}

//==============================================================================
// EventSelect
//==============================================================================
bool MCAna::EventSelect() {
  return true;
}

//==============================================================================
// DefinePattern
//==============================================================================
bool MCAna::DefinePattern() {
  
  PiPlus  = pi_plus.createPattern();
  PiMinus = pi_minus.createPattern();

  K_short = K_S.decaysTo(PiPlus & PiMinus);

  PiMinus->with(pt > 0.2);
  PiPlus->with(pt > 0.1);
  K_short->with(pt > 0.15);

  cout << "K_short: " << endl << *K_short->pattern() << endl;
  return true;
}

//==============================================================================
// DefineHistograms
//==============================================================================
bool MCAna::DefineHistograms() {
  return true;
}

//==============================================================================
// Event
//==============================================================================
bool MCAna::Event() {

  PiPlus->clear();
  PiMinus->clear();

  // example on how to loop manually over MC_Track and print decay chains.
  unsigned int nk0s = 0;

  const ArteTableIterator<MC_Track> end = ArteTable<MC_Track>::end();
  for( ArteTableIterator<MC_Track> it = ArteTable<MC_Track>::begin();
       it != end; ++it ) {
    const ArtePointer<MC_Track> mtr = *it;
    if(mtr->bVertex().null())
      cout << "Warning: no Begin Vertex available!" << endl;

    if(*mtr == K_S) {
      mtr->print_decay(0,'r');
      cout << endl;
      ++nk0s;
    }
  }
  cout << "No of K_zero's: " << nk0s << endl;


  // MC decay chain selection deluxe:
  K_short->selectMCtruth();

  cout << "K_short size: " << K_short->matches().size() 
       << " PiPlus size: " << PiPlus->matches().size()
       << " PiMinus size: " << PiMinus->matches().size() 
       << endl;

  cout << "PiPlus container: " << endl;
  cout << PiPlus;
  cout << "PiMinus container: " << endl;
  cout << PiMinus;

  cout << "K_short container: " << endl;
  cout << K_short;
  return true;
}

//==============================================================================
// EventCleanup
//==============================================================================
bool MCAna::EventCleanup() {
  return true;
}

//==============================================================================
// End processing
//==============================================================================
bool MCAna::End() {
  return true;
}

