#ifndef ARTESPECIALIZE_HH
#define ARTESPECIALIZE_HH
// ********************************************************************
//
// source:
//
// type:      source code
//
// created:   27. Feb 1999
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: Preprocessor Macro definitions for ARTE specialization.
//              This adopts the ArteTable, ArtePointer templates to ROOT containers.
//              This file was formerly called "ClonesTable.icc"
//
// changes:
// 27 Feb 1999 creation by C. Borgmeier
// 14 Nov 2001 (TG) renamed from ClonesTable.icc to ArteSpecialize.hh
//
// ********************************************************************

// #include "arte/arte.hh"
#undef getLink

#include "ClonesArray.hh"

#ifdef USE_ROOT

#define ArtePointerSpecialize(T) \
template<>\
inline int* ArtePointer<T>::getBase()\
{\
   return static_cast<int*>( ArteTable<T>::getLink() );\
}\
\
\
template<>\
inline void* ArtePointer<T>::dereference() const\
{\
   TClonesArray* array = static_cast<TClonesArray*>( ArteTable<T>::getLink() );\
 return array->At(index()-1);\
}\
\
\
template<>\
inline unsigned int ArteTable<T>::size()\
{\
   CLUE::GClonesArray<T*>* array = static_cast<CLUE::GClonesArray<T*>*>( getLink() );\
 return array->size();\
}\
\
\
template<>\
inline ArtePointer<T> ArteTable<T>::push_back(const T& e)\
{\
   CLUE::GClonesArray<T*>* array = static_cast<CLUE::GClonesArray<T*>*>( getLink() );\
 array->push_back( const_cast<T*>(&e) );\
 return *(end()-1);\
}\
\
\
template<>\
inline bool ArteTable<T>::isArteTable()\
{\
   return false;\
}\
\
template<>\
inline void ArteTable<T>::Print( Option_t* )\
{\
  CLUE::GClonesArray<T*>* array = static_cast<CLUE::GClonesArray<T*>*>( getLink() );\
  cout << "Print a table: " << array->ClassName() << endl;\
  array->print(std::cout);\
}

#else /* nearly the same */


#define ArtePointerSpecialize(T) \
template<>\
inline unsigned int ArteTable<T>::size()\
{\
 CLUE::GClonesArray<T*>* array = static_cast<CLUE::GClonesArray<T*>*>( getLink() );\
 return array->size();\
}\
\
\
template<>\
inline ArtePointer<T> ArteTable<T>::push_back(const T& e)\
{\
 CLUE::GClonesArray<T*>* array = static_cast<CLUE::GClonesArray<T*>*>( getLink() );\
 array->push_back( const_cast<T*>(&e) );\
 return *(end()-1);\
}\
\
template<>\
inline bool ArteTable<T>::isArteTable()\
{\
   return false;\
}\

#undef getLink

#endif

#endif

