#ifndef GEOM_INI_HH
#define GEOM_INI_HH
// *****************************************************************************
//
// source:
//
// type:      source code
//
// created:   20. Mai 2000
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: Class to initialize the Geometry package
//
// changes:
// 20 Mai 2000 (TG) creation
// 01 Nov 2000 (TG) inherited GeomIni from TObject, full ROOT integration
// 20 Nov 2000 (TG) added namespace CLUE
//
// *****************************************************************************
#include <iosfwd>

#include "arte/TObject.hh"

#ifdef NAMESPACE
namespace CLUE {
#endif

  /**
     The initialization of the package is done automatically when the
     shared library is loaded into memory. An instance should only be created to
     obtain the information provided by the member functions.
     @memo Initialization of Geometry package
     @author T. Glebe
  */
  //==============================================================================
  // GeomIni
  //==============================================================================
  class GeomIni : public TObject {
  public:
    /// constructor
    GeomIni();

    /// get string with version number
    inline static const char* const version() { return Version; }
    /// get string with package name
    inline static const char* const name()    { return Name; }
    /// get birthday \& time of package version
    inline static const char* const date()    { return Date; }
    
    /** @name --- Expert functions --- */
    /// used by operator<<()
    void print(std::ostream& os) const;

  private:
#ifndef __CINT__
    /// not available in CINT
    class initializer {
    public:
      initializer();
    };
    
    /// constructor call for initalization
    static initializer doIni; 
#endif
    static const char *Version;
    static const char *Name;
    static const char *Date;
    
#ifdef USE_ROOT
    ClassDef(GeomIni,1)
#endif
  }; // end of class GeomIni


  //============================================================================
  // operator<<
  //============================================================================
  inline std::ostream& operator<< ( std::ostream& os, const GeomIni& t ) {
    t.print(os);
    return os;
  }

#ifdef NAMESPACE
} // namespace CLUE
#endif
#endif
