#ifndef __ANALYSISIF_HH
#define __ANALYSISIF_HH
// ********************************************************************
//
// source:
//
// type:      source code
//
// created:   28. Feb 2001
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: Analysis interface class
//
// changes:
// 28 Feb 2001 (TG) creation
// 12 Nov 2001 (TG) comments improved
//
// ********************************************************************

/** Abstract class which defines the minimal interface for an Analysis object

    @memo Analysis Interface
    @author T. Glebe
*/
//==============================================================================
// AnalysisIf
//==============================================================================
class AnalysisIf {
public:
  /** @name --- Access methods --- */
  /// call event selection (\texttt{true} = accept event)
  virtual bool EventSelect() = 0;
  /// call routine for histogram booking
  virtual bool DefineHistograms() = 0;
  /// call routine for \emph{Pattern} definitions
  virtual bool DefinePattern() = 0;
  /// call event processing routine
  virtual bool Event() = 0;
  /// call cleanup routine at end of event
  virtual bool EventCleanup() = 0;
  /// call cleanup routine at end of job
  virtual bool End() = 0;
  /// return name of analysis class
  virtual const char* name() const = 0;
}; // end of AnalysisIf
#endif
