#ifndef __TOPPIF__HH
#define __TOPPIF__HH
// ********************************************************************
//
// source:
//
// type:      source code
//
// created:   24. Jan 2001
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: Particle Data interface class
//
// changes:
// 24 Jan 2001 (TG) creation
// 29 Jan 2001 (TG) added anti_chrg(), anti_name()
// 28 Feb 2001 (TG) operator==() added
// 22 Aug 2001 (TG) added print(), operator<<()
// 05 Sep 2001 (TG) added lund()
// 12 Nov 2001 (TG) comment improved
//
// ********************************************************************

/** Abstract class which defines the minimal interface to Topp Table
    entries. This interface decouples the classes Topp and GTopp.

    @memo Particle Data Interface
    @author T. Glebe
*/
//==============================================================================
// ToppIf
//==============================================================================
class ToppIf {
public:

  /** @name --- Access methods --- */
  /// particle name
  virtual const char* const name()  const = 0;
  /// mass
  virtual float             mass()  const = 0;
  /// width
  virtual float             wid()   const = 0;
  /// truncate Breit-Wigner at mass - trbw
  virtual float             trbw()  const = 0;
  /// life time
  virtual float             life()  const = 0;
  /// 3 * charge
  virtual int               chrg()  const = 0;
  /// Do we represent a valid Topp entry?
  virtual bool              valid() const = 0;
  /// access to anti-particle information
  virtual const ToppIf&     anti()  const = 0;
  /// LUND particle code
  virtual int               lund()  const = 0;
  
  /** @name --- Expert functions --- */
  /// used by operator<<()
  virtual std::ostream& print( std::ostream& ) const = 0;

  /** @name --- Operators --- */
  ///
  virtual bool operator==(const ToppIf& rhs) const = 0;
}; // end of ToppIf

//==============================================================================
// operator <<
//==============================================================================
inline std::ostream& operator<< (std::ostream& os, const ToppIf& t) {
  return t.print(os);
}
#endif
