#ifndef __PROPERTYIF_HH
#define __PROPERTYIF_HH
// ********************************************************************
//
// source:
//
// type:      source code
//
// created:   28. Feb 2001
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: Property object interface
//
// changes:
// 28 Feb 2001 (TG) creation
//
// ********************************************************************
#include <iosfwd>

/** Abstract class which defines the minimal interface for a Property.

    @author T. Glebe
*/
//==============================================================================
// PropertyIf
//==============================================================================
class PropertyIf {
public:
  /** @name --- Expert methods --- */
  template <class C, class T>
  T evaluate( const C& ) const;
  /// Property factory function
  virtual PropertyIf* propertyClone() const = 0;
  /// used by operator<<()
  virtual std::ostream& print( std::ostream& ) const = 0;
}; // end of class PropertyIf


//==============================================================================
// operator <<
//==============================================================================
inline std::ostream& operator<< (std::ostream& os, const PropertyIf& t) {
  return t.print(os);
}
#endif
