// ********************************************************************
//
// source:
//
// type:      source code
//
// created:   28. Jun 2001
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: A PatternMatch container class
//
// changes:
// 28 Jun 2001 (TG) creation
// 28 Jun 2001 (TG) removed template parameter C and T
// 22 Aug 2001 (TG) added with()
// 28 Aug 2001 (TG) changed push_back(TrackIf) to push_back(MC_Track)
// 05 Nov 2001 (TG) added push_back(TrackIf)
// 12 Dec 2001 (TG) bug fix in select(), moved select() to .icc file
// 14 Jan 2002 (TG) added ownership argument to push_back(RecoSegment),
//                  changes because pmlist stores now SmartPointer
//
// ********************************************************************
#include "pattern/SPatternMatchContainer.hh"

//==============================================================================
// Constructors
//==============================================================================
SPatternMatchContainer::SPatternMatchContainer() : 
  pmlist(),
  ptrlist(),
  pattern_(SmartPointer<SPattern>())
{}

SPatternMatchContainer::SPatternMatchContainer(const SPattern& rhs) : 
  pmlist(),
  ptrlist(),
  pattern_(SmartPointer<SPattern>(rhs))
{ pattern_->set_container(*this); }

SPatternMatchContainer::SPatternMatchContainer(const SmartPointer<SPattern>& rhs) : 
  pmlist(),
  ptrlist(),
  pattern_(rhs)
{ pattern_->set_container(*this); }

SPatternMatchContainer::~SPatternMatchContainer() {}

//==============================================================================
// push_back
//==============================================================================
ParticleCandidate SPatternMatchContainer::push_back(RecoSegment& rs, bool ownership)  { 

  pmlist.push_back(ParticleCandidate( SPatternMatch(rs, pattern_) )); 
  ParticleCandidate pm = pmlist.back();
  pm->set_ownership(ownership);
  ptrlist.push_back(&(*pm)); 
  return pm;
}

ParticleCandidate SPatternMatchContainer::push_back(MC_Track& mtra) { 

  pmlist.push_back(ParticleCandidate( SPatternMatch(mtra, pattern_) ));
  ParticleCandidate pm = pmlist.back();
  ptrlist.push_back(&(*pm)); 
  return pm;
}

ParticleCandidate SPatternMatchContainer::push_back(TrackIf& t) { 

  pmlist.push_back(ParticleCandidate( SPatternMatch(t, pattern_) ));
  ParticleCandidate pm = pmlist.back();
  ptrlist.push_back(&(*pm)); 
  return pm;
}

//==============================================================================
// with
//==============================================================================
void SPatternMatchContainer::with(const SProperty<TrackIf,bool>& rhs)
{ pattern_->setProperty(rhs); }


//==============================================================================
// print
//==============================================================================
std::ostream& SPatternMatchContainer::print(std::ostream& os) const {
  const SPM_cit end = pmlist.end();
  for(SPM_cit it = pmlist.begin(); it != end; ++it) {
    os << **it << endl;
  }
  //  copy(pmlist.begin(), pmlist.end(),
  //       ostream_iterator<SmartPointer<SPatternMatch> >(os, "\n"));
  return os;
}
