// ********************************************************************
//
// source:
//
// type:      source code
//
// created:   29. Jun 2001
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: Track Interface Property objects
//
// changes:
// 28 Jun 2001 (TG) creation
// 29 Jun 2001 (TG) added charge, pz, energy, xf, rap
//
// ********************************************************************
#include "interfaces/TrackIf.hh"
#include "pattern/SProperty.hh"

//==============================================================================
// TrackIf Pattern elements
//==============================================================================
const SPMember<TrackIf,float> x      ( &TrackIf::x , "x"  );
const SPMember<TrackIf,float> y      ( &TrackIf::y , "y"  );
const SPMember<TrackIf,float> z      ( &TrackIf::z , "z"  );

const SPMember<TrackIf,float> tx     ( &TrackIf::tx, "tx" );
const SPMember<TrackIf,float> ty     ( &TrackIf::ty, "ty" );
const SPMember<TrackIf,float> p      ( &TrackIf::p , "p"  );
const SPMember<TrackIf,float> pt     ( &TrackIf::pt, "pt" );
const SPMember<TrackIf,float> pz     ( &TrackIf::pz, "pz" );
const SPMember<TrackIf,float> chi2   ( &TrackIf::chi2, "chi2" );
const SPMember<TrackIf,unsigned short int> ndf   ( &TrackIf::ndf, "ndf" );

const SPMember<TrackIf,float> azim   ( &TrackIf::phi, "phi" );
const SPMember<TrackIf,float> theta  ( &TrackIf::theta, "theta" );
const SPMember<TrackIf,float> psrap  ( &TrackIf::eta, "eta" );

const SPMember<TrackIf,int>   charge ( &TrackIf::charge, "charge" );

const SPMemberOnePar<TrackIf,float,double> energy ( &TrackIf::energy, 0., "energy" );
const SPMemberOnePar<TrackIf,float,double> xf     ( &TrackIf::xf, 0., "xf" );
const SPMemberOnePar<TrackIf,float,double> rap    ( &TrackIf::rap, 0., "rap" );

const SPConstant<TrackIf,bool> all    ( true );
const SPMember<TrackIf,bool>   isValid ( &TrackIf::isValid, "isValid" );
