#ifndef P_TRAIT_HH
#define P_TRAIT_HH
// *****************************************************************************
//
// source:
//
// type:      source code
//
// created:   25. Jun 2001
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: trait to configure SPattern classes
//
// changes:
// 25 Jun 2001 (TG) creation
// 27 Jun 2001 (TG) added #include GCombiner.hh
// 28 Jun 2001 (TG) remove P_trait
// 02 Jul 2001 (TG) added typedefs for [Decay]PatternMatch pointer
// 12 Dec 2001 (TG) added TwoProngDecay,... typedefs
// 04 Jan 2002 (TG) added TwoProngCandidate,... typedefs
// 07 Jan 2002 (TG) added ParticleCandidate typedefs
// 14 Jan 2002 (TG) changed SPM_l and SDPM_l to SmartPointer types, changed
//                  also ParticleCandidate and *ProngCandidate typedefs to
//                  SmartPointer types
//
// *****************************************************************************

#include <list>

#ifndef __CINT__
#include "gcombiner/GSCombiner.hh"
#endif

class TrackIf;
class SPattern;
class SPatternMatch;
class PatternMatchIf;
class SPatternMatchContainer;
template <class C, class T> class SProperty;
template <class T> class SmartPointer;
template <unsigned int NTR> class SDecayPattern;
template <unsigned int NTR> class SDecayPatternMatch;
template <unsigned int NTR> class SDecayPatternMatchContainer;
template <unsigned int NTR> class SPatternExpression;


//============================================================================
// Some global typedefs
//============================================================================
/** @name Pattern Match typedefs.
    @author T. Glebe
 */
//@{
///
typedef list<SmartPointer<SPatternMatch> >   SPM_l;
///
typedef SPM_l::iterator                      SPM_it;
///
typedef SPM_l::const_iterator                SPM_cit;
///
typedef SmartPointer<SPatternMatch>          ParticleCandidate;
///
typedef SPM_it                               ParticleCandidateIterator;
///
typedef SPM_cit                              ParticleCandidateConstIterator;

///
typedef list<PatternMatchIf*>                SPMP_l;
///
typedef SPMP_l::iterator                     SPMP_it;
///
typedef SPMP_l::const_iterator               SPMP_cit;

#ifndef __CINT__
///
typedef GSCombiner<SPMP_l>                   GSC;
//typedef GSCombiner<list<TrackIf*> >                   GSC;
///
typedef GSC::iterator                        GSC_it;
#endif

#ifdef XXX
///
typedef GCombiner<SPM_l>                    PMC;
///
typedef GCombiner<SPM_l>::iterator          PMC_it;
#endif

///
typedef SmartPointer<SPattern>               SP_sp;
///
typedef SmartPointer<SPatternMatchContainer> PMC_sp;
///
typedef SmartPointer<SPatternMatchContainer> TrackPattern;

//@}

/** @name Decay Pattern trait

    Configuration of types used in the DecayPatternMatch classes.

    @author T. Glebe
 */
//@{
//============================================================================
// The DP-Trait (for SDecayPatternMatches)
//============================================================================
/** Trait class for SDecayPattern. */
template <unsigned int NTR>
struct DP_trait {

  ///
  typedef list<SmartPointer<SDecayPatternMatch<NTR> > >      SDPM_l;
  ///
  typedef SDPM_l::iterator                    SDPM_it;
  ///
  typedef SDPM_l::const_iterator              SDPM_cit;

  ///
  typedef list<SDecayPatternMatch<NTR>*>      SDPMP_l;
  ///
  typedef SDPMP_l::iterator                   SDPMP_it;
  ///
  typedef SDPMP_l::const_iterator             SDPMP_cit;

  ///
  typedef SmartPointer<SDecayPattern<NTR> >   SDP_sp;

  /// Container for DecayPatternMatch objects
  typedef SmartPointer<SDecayPatternMatchContainer<NTR> > DPMC_sp;
}; // end of DP_trait
//@}

/** @name Decay Pattern typedefs.
    @author T. Glebe
 */
//@{
/// 2-prong Decay Pattern
typedef DP_trait<2>::DPMC_sp                 TwoProngDecay;
/// DecayPatternMatch iterator
typedef DP_trait<2>::SDPM_it                 TwoProngCandidateIterator;
/// DecayPatternMatch const iterator
typedef DP_trait<2>::SDPM_cit                TwoProngCandidateConstIterator;
/// DecayPatternMatch
typedef SmartPointer<SDecayPatternMatch<2> > TwoProngCandidate;
/// 3-prong Decay Pattern
typedef DP_trait<3>::DPMC_sp                 ThreeProngDecay;
/// DecayPatternMatch iterator
typedef DP_trait<3>::SDPM_it                 ThreeProngCandidateIterator;
/// DecayPatternMatch const iterator
typedef DP_trait<3>::SDPM_cit                ThreeProngCandidateConstIterator;
/// DecayPatternMatch
typedef SmartPointer<SDecayPatternMatch<3> > ThreeProngCandidate;
/// 4-prong Decay Pattern
typedef DP_trait<4>::DPMC_sp                 FourProngDecay;
/// DecayPatternMatch iterator
typedef DP_trait<4>::SDPM_it                 FourProngCandidateIterator;
/// DecayPatternMatch const iterator
typedef DP_trait<4>::SDPM_cit                FourProngCandidateConstIterator;
/// DecayPatternMatch
typedef SmartPointer<SDecayPatternMatch<4> > FourProngCandidate;
/// 5-prong Decay Pattern
typedef DP_trait<5>::DPMC_sp                 FiveProngDecay;
/// DecayPatternMatch iterator
typedef DP_trait<5>::SDPM_it                 FiveProngCandidateIterator;
/// DecayPatternMatch const iterator
typedef DP_trait<5>::SDPM_cit                FiveProngCandidateConstIterator;
/// DecayPatternMatch
typedef SmartPointer<SDecayPatternMatch<5> > FiveProngCandidate;
/// 6-prong Decay Pattern
typedef DP_trait<6>::DPMC_sp                 SixProngDecay;
/// DecayPatternMatch iterator
typedef DP_trait<6>::SDPM_it                 SixProngCandidateIterator;
/// DecayPatternMatch const iterator
typedef DP_trait<6>::SDPM_cit                SixProngCandidateConstIterator;
/// DecayPatternMatch
typedef SmartPointer<SDecayPatternMatch<6> > SixProngCandidate;
//@}
#endif
