#ifndef SPATTERNEXPRESSION_HH
#define SPATTERNEXPRESSION_HH
// ********************************************************************
//
// source:
//
// type:      source code
//
// created:   26. Jun 2001
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: [Decay]PatternMatchContainer expression class
//
// changes:
// 26 Jun 2001 (TG) creation
// 28 Jun 2001 (TG) removed template parameter C and T
// 02 Jul 2001 (TG) changed type of pmlist to SPMP_l
// 04 Jul 2001 (TG) operator& added, CTOR added
// 17 Jul 2001 (TG) operator& added, CTOR added
// 18 Dec 2001 (TG) modef operator&() definitions to .icc file
//
// ********************************************************************
#include <iosfwd>
#include "pattern/P_trait.hh"
#include "pattern/SPatternMatchContainer.hh"

class ToppIf;
template <unsigned int NTR> class SDecayPattern;

/** Class which represents a (Decay)PatternMatchContainer expression.

    @memo For internal use only.
    @author T. Glebe
*/
//==============================================================================
// SPatternExpression
//==============================================================================
template <unsigned int NTR>
class SPatternExpression {

  typedef typename DP_trait<NTR>::SDP_sp         SDP_sp;

public:
  /** @name --- Constructors --- */
  ///
  SPatternExpression();
  ///
  SPatternExpression(const SDecayPattern<NTR>& dp,
		     const PMC_sp& lhs, const PMC_sp& rhs);
  ///
  template <unsigned int NTR2>
  SPatternExpression(const SDecayPattern<NTR>& dp,
		     const PMC_sp& lhs, const
		     SmartPointer<SDecayPatternMatchContainer<NTR2> >& rhs);
  ///
  SPatternExpression(const SDP_sp& dp,
		     const PMC_sp& lhs, const PMC_sp& rhs);
  ///
  SPatternExpression(const ToppIf& topp,
		     const SPatternExpression<NTR>& rhs);
  ///
  SPatternExpression(const SDecayPattern<NTR>& dp,
		     const SPatternExpression<NTR-1>& lhs,
		     const PMC_sp& rhs);
  ///
  template <unsigned int NTR2>
  SPatternExpression(const SDecayPattern<NTR>& dp,
		     const SPatternExpression<NTR-1>& lhs, const
		     SmartPointer<SDecayPatternMatchContainer<NTR2> >& rhs);

  ///
  template <unsigned int NTR1, unsigned int NTR2>
  SPatternExpression(const SDecayPattern<NTR>& dp,
                     const SmartPointer<SDecayPatternMatchContainer<NTR1> >& lhs,
                     const SmartPointer<SDecayPatternMatchContainer<NTR2> >& rhs);
  
  /** @name --- Access methods --- */
  /// get decay pattern
  const SDecayPattern<NTR>& dpattern() const;
  ///
  const SPMP_l* pmList(unsigned int i) const;

private:
  const SPMP_l* pmlist[NTR]; // input: pattern match lists
  SDP_sp        dpattern_;   // the decay pattern
}; // end of SPatternExpression


//==============================================================================
// operator&
//==============================================================================
SPatternExpression<2> operator&(const PMC_sp& lhs, const PMC_sp& rhs);

template <unsigned int NTR>
SPatternExpression<NTR> operator&( const SPatternExpression<NTR-1>& lhs,
				   const PMC_sp& rhs);

template <unsigned int NTR2>
SPatternExpression<2> operator&(const PMC_sp& lhs,
				const SmartPointer<SDecayPatternMatchContainer<NTR2> >& rhs);

template <unsigned int NTR2>
SPatternExpression<2> operator&( const SmartPointer<SDecayPatternMatchContainer<NTR2> >& lhs,
				 const PMC_sp& rhs);

template <unsigned int NTR, unsigned int NTR2>
SPatternExpression<NTR> operator&( const SPatternExpression<NTR-1>& lhs,
				   const SmartPointer<SDecayPatternMatchContainer<NTR2> >& rhs);

template <unsigned int NTR>
SPatternExpression<2> operator&( const SmartPointer<SDecayPatternMatchContainer<NTR> >& lhs,
				 const SmartPointer<SDecayPatternMatchContainer<NTR> >& rhs);

template <unsigned int NTR1, unsigned int NTR2>
SPatternExpression<2> operator&( const SmartPointer<SDecayPatternMatchContainer<NTR1> >& lhs,
				 const SmartPointer<SDecayPatternMatchContainer<NTR2> >& rhs);

#include "pattern/SPatternExpression.icc"
#endif
