#ifndef __GPATTERNSELECT_HH
#define __GPATTERNSELECT_HH
// ********************************************************************
//
// source:
//
// type:      source code
//
// created:   20. Jul 1997
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: routines for selecting tracks by a pattern
//
// changes:
// 26 Jan 2001 (TG) creation
//
// ********************************************************************
#include <algorithm>
#include "pattern/Pattern.hh"
#include "pattern/PatternMatch.hh"

//==============================================================================
// select: a track selection routine
//==============================================================================
// GPatternMatch factory function!
// Assumption: Source is Container of C*! (because of GPattern::evaluate)
// Assumption: Target is container of GPatternMatch<C,T>*!
/// gselect adoption for the PatternMatch function
template <class Source, class Target>
bool select(const Source& src, Target& trg, const Pattern& pat) {
  typedef typename Source::const_iterator Cit;

  const Cit end = src.end();
  for(Cit it = src.begin(); it != end; ++it) {
    const typename Source::value_type& track = *it;
    if( pat.evalTrack(*track) ) {
      trg.push_back( new PatternMatch( *track, pat ) );
    }
  }

  return true;
}
#endif
