#ifndef __DECAYPATTERN_HH
#define __DECAYPATTERN_HH
// ********************************************************************
//
// source:
//
// type:      source code
//
// created:   01. Feb 2001
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: ROOT specific class describing a decay pattern
//              for RecoTracks
//
// changes:
// 01 Feb 2001 (TG) creation
//
// ********************************************************************
#include <iosfwd>
#include "arte/TObject.hh"
#include "pattern/GDecayPattern.hh"

namespace VERTEX { class Vertex; }
class DecayPatternMatch;
class ToppIf;
class Pattern;

typedef GDecayPattern<RecoTrack,int>                    DPatternBase;
typedef ClProperty<VERTEX::Vertex,int>                  VtxProp;
typedef ClProperty<VERTEX::Vertex,double>               VtxProp2;

typedef list<DecayPatternMatch*>                 DPM_l;
typedef list<DecayPatternMatch*>::iterator       DPM_it;
typedef list<DecayPatternMatch*>::const_iterator DPM_cit;
typedef ArteTable<RecoTrack>                     RT_t;

/** DecayPattern is a specialization of the GDecayPattern class for RecoTrack
    objects. DecayPattern is integrated into the ROOT RTTI system.

    @memo ROOT class
    @author T. Glebe
 */
//==============================================================================
// DecayPattern
//==============================================================================
class DecayPattern : public TObject, public DPatternBase {
public:
  /** @name --- Constructors --- */
  ///
  DecayPattern();
  ///
  DecayPattern( const DPatternBase& rhs);
  ///
  DecayPattern( const ToppIf& tp, const DPatternBase& rhs);
  ///
  DecayPattern( const ToppIf& tp, const DecayPattern& rhs);
  ///
  DecayPattern( const DecayPattern& rhs);
  ///
  DecayPattern( const ToppIf& tp, const Pattern& lhs, const Pattern& rhs);
  ///
  ~DecayPattern();

  /** @name --- Operators --- */
  ///
  DecayPattern& operator&( const Pattern& rhs);
  /// put everything into a single vertex (to be improved)
  DecayPattern& operator&( const DecayPattern& rhs);

  /// forwarding function
  bool select(const RT_t& src, DPM_l& trg);

  /// forwarding function
  inline void with(const VtxProp& rhs) {
    DPatternBase::with(rhs);
  }

  /// dummy function for right ROOT instantiation order
  void with(const VtxProp2& rhs) const;

#ifdef USE_ROOT
  ClassDef(DecayPattern,1)
#endif
}; // end of DecayPattern

//==============================================================================
// operator <<
//==============================================================================
inline std::ostream& operator<<( std::ostream& os, const DecayPattern& a ) {
  return a.print( os );
}

#endif
